cvs_id = "$Id: MainWindow.py,v 1.68.2.2 2003/08/31 03:30:59 jmalonzo Exp $"

import StringIO
import re

import gobject
import gtk
import gnome
import gtkhtml2
import gnome.ui
import pango
import string
from gtk import glade

import straw
from error import log
import htmlentitydefs

class ExceptionView:
    def __init__(self, frame, label):
        self._frame = frame
        self._label = label

    def show_exception(self, exception):
        self._label.set_text(
            _("<b>An error occured while reading this feed:</b>\n%s" %
              str(exception)))
        self._label.set_property("use_markup", True)
        self._frame.show()
        return

    def hide(self):
        self._frame.hide()
        return

class DummyDocStream(StringIO.StringIO):
    def write_stream(self, s):
        self.write(s)

url_re = re.compile('^\w+://')
def is_url(text):
    return url_re.match(text)

class ItemView:
    def __init__(self, item_view_container):
        widget_tree = glade.get_widget_tree(item_view_container)

        self._source_display_frame = widget_tree.get_widget(
            'source_display_frame')
        self._source_name_label = widget_tree.get_widget('source_name_label')
        self._source_url_label = widget_tree.get_widget('source_url_label')
        widget_tree.signal_connect('on_source_subscribe_button_clicked',
                                   self.on_source_subscribe_button_clicked)

        scrolled_window = widget_tree.get_widget('html_scrolled_window')
        htmlview = gtkhtml2.View()
        self._document = gtkhtml2.Document()
        self._document.connect("link-clicked", self.link_clicked)
        htmlview.connect("on_url", self.on_url)
        self._document.connect("request-url", self.request_url)
        htmlview.get_vadjustment().set_value(0)
        scrolled_window.set_hadjustment(htmlview.get_hadjustment())
        scrolled_window.set_vadjustment(htmlview.get_vadjustment())
        self._document.clear()
        htmlview.set_document(self._document)
        scrolled_window.add(htmlview)
        scrolled_window.show_all()
        self._scrolled_window = scrolled_window
        self._item = None
        self._htmlview = htmlview

    def on_url(self, view, url):
        if url == None:
            url = ""
        if len(url):
            url = straw.main.get_visible_feed().complete_url(url)
        straw.main_window.display_status_message(url)
        return

    def link_clicked(self, document, link):
        link = link.strip()
        link = straw.main.get_visible_feed().complete_url(link)
        gnome.url_show(link)
        return

    def request_url(self, document, url, stream):
        try:
            image = straw.ImageCache.retrieve(
                straw.main.get_visible_feed().complete_url(url))
            stream.write(image.get_data())
            stream.close()
        except Exception, ex:
            stream.close()

    def on_source_subscribe_button_clicked(self, button, *data):
        f = straw.Feed(self._item.source['text'],
                       location = self._item.source['url'])
        straw.config.feeds.append(f)
        straw.poll_manager.poll(f)
        return

    def display_item(self, item):
        self._document.clear()
        self._document.open_stream("text/html")
        enc = None
        html = (
            """<html><head>
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
            <style type="text/css">
            q { font-style: italic;}
            blockquote { display: block; font-style: italic; }
            .stitle {background-color:transparent;font-size:small;font-weight:bold;}
            .sdate {font-size:small}
            .content {padding-left:10px;margin-top:10px}
            </style>
            <title>title</title></head><body>""" +
                self.htmlify_item(item) +
                "</body></html>")
        html = html.encode('utf-8')
        self._document.write_stream(html)
        self._document.close_stream()
        va = self._scrolled_window.get_vadjustment()
        va.set_value(va.lower)
        if item.source is not None:
            self._source_name_label.set_text(item.source['text'])
            self._source_url_label.set_text(item.source['url'])
            self._source_display_frame.show()
        else:
            self._source_display_frame.hide()
        self._item = item
        return

    def display_empty_feed(self, feed):
        self._document.clear()
        self._document.open_stream("text/html")
        self._document.write_stream(self.htmlify_empty_feed(feed))
        self._document.close_stream()
        self._item = None
        self._source_display_frame.hide()
        return

    def htmlify_empty_feed(self, feed):
        return ("<html><head><title>title</title></head><body><p>" +
                   _("No data yet, need to poll first.") +
                "</p></body></html>")

    def htmlify_item(self, item):
        ret = []
        if item.title is not None:
            ret.append('<div class="stitle">%s</div>' % item.title)
        if item.pub_date is not None:
            format = '%a, %d %b %H:%M:%S'
            ret.append('<div class="sdate">%s</div>' %
                       unicode(item.pub_date.localtime().strftime(format),'latin-1').encode('utf-8'))
        ret.append('<div class="content">')
        if item.description is not None:
            ret.append('%s ' % item.description)
        ret.append('</div>')
        if item.creator is not None:
            ret.append('<p>%s: %s</p>' % (_("Creator"), item.creator))
        if item.fm_license is not None:
            ret.append('<p>%s: %s</p>' % (_("Software license"),
                                          item.fm_license))
        if item.fm_changes is not None:
            ret.append('<p>%s: %s</p>' % (_("Changes"), item.fm_changes))
        ret.append("<p>")
        if item.license_urls is not None:
            for l in item.license_urls:
                ret.append('<a href="%s">%s</a> ' % (l, _("License")))
        ret.append('</p><p>')
        if item.publication_name is not None:
            ret.append('%s: %s ' % (_("Publication"),
                                    item.publication_name))
            if item.publication_volume is not None:
                ret.append('%s' % item.publication_volume)
                if item.publication_number is not None:
                    ret.append('(%s)' % item.publication_number)
            if item.publication_section is not None:
                ret.append(' %s' % item.publication_section)
            if item.publication_starting_page is not None:
                ret.append(', %s %s' % (_("Page"),
                                        item.publication_starting_page))
            ret.append('</p><p>')
        if item.link is not None:
            ret.append('<a href="%s">%s &gt;&gt;</a><br />' %
                       (item.link,_("Full Story")))
        if item.guid is not None and is_url(item.guid):
            ret.append('<a href="%s">%s &gt;&gt;</a>' %
                       (item.guid, _("Permanent link")))
        ret.append('</p>')
        return "".join(ret)

    def scroll_down(self):
        va = self._scrolled_window.get_vadjustment()
        old_value = va.get_value()
        new_value = old_value + va.page_increment
        limit = va.upper - va.page_size
        if new_value > limit:
            new_value = limit
        va.set_value(new_value)
        return new_value > old_value

def read_text(fragment, chars):
    """Read chars cdata characters from html fragment fragment"""
    count = 0
    in_tag = 0
    in_attr = 0
    in_entity = 0
    res_len = 0
    if fragment is None:
        return ""
    frag_len = len(fragment)
    i = -1
    res = []
    previous = None
    while res_len < chars and i < frag_len - 1:
        i += 1
        current = fragment[i]
        if in_tag:
            if in_attr:
                if current == '"':
                    in_attr = 0
            else:
                if current == '"':
                    in_attr = 1
                elif current == '>':
                    in_tag = 0
        elif in_entity:
            res.append(current)
            if current == ';':
                in_entity = 0
                previous = ""
                res_len += 1
        else:
            if current == "<":
                in_tag = 1
            elif current == "&":
                in_entity = 1
                res.append("&")
            # avoid beginning whitespace, newlines and returns, and multiple
            # whitespaces
            elif len(res) or current not in string.whitespace:
                if current == '\n' or current == '\r':
                    current = ' '
                    if current == previous == ' ':
                        continue
                res.append(current)
                previous = current
                res_len += 1
    return "".join(res)

entity = re.compile(r'\&.\w*?\;')
def convert_entities(text):
    def conv(ents):
        entities = htmlentitydefs.entitydefs
        ents = ents.group(0)
        ent_code = entities.get(ents[1:-1], None)
        if ent_code is not None:
            return unicode(ent_code, 'iso8859-1')
        else:
            if ents[1] == '#':
                code = ents[2:-1]
                base = 10
                if code[0] == 'x':
                    code = code[1:]
                    base = 16
                return unichr(int(code, base))
            else:
                return

    in_entity = entity.search(text)
    if in_entity is None:
        return text
    else:
        # convert all entities found in text then return the converted text
        ctext = in_entity.re.sub(conv, text)
        return ctext

html_tags = re.compile(r'\<.*?\>')
def is_html(text):
    is_tag = html_tags.search(text)
    if is_tag: return 1
    else: return 0

class ItemList:
    COLUMN_TITLE = 0
    COLUMN_STICKY = 1
    COLUMN_ITEM = 2
    COLUMN_BOLD = 3
    COLUMN_STICKY_FLAG = 4

    def __init__(self, widget):
        self._widget = widget
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_OBJECT,
                              gobject.TYPE_PYOBJECT, gobject.TYPE_INT,
                              gobject.TYPE_BOOLEAN)
        widget.set_model(model)
        widget.set_rules_hint(gtk.TRUE)
        self.create_item_selection_columns(self._widget)
        widget.get_selection().connect("changed", self.item_selection_changed)
        self._feed = None
        return

    def create_item_selection_columns(self, item_list_view):
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Title'), renderer,
                                    text=self.COLUMN_TITLE,
                                    weight=self.COLUMN_BOLD)
        column.set_min_width(int(550))
        item_list_view.append_column(column)

        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_('Keep'), renderer,
                                    active=self.COLUMN_STICKY_FLAG)
        column.set_max_width(int(50))
        item_list_view.append_column(column)
        renderer.connect('toggled', self.sticky_toggled)
        return

    def sticky_toggled(self, cell, path):
        model = self._widget.get_model()
        iter = model.get_iter((int(path),))
        item = model.get_value(iter, self.COLUMN_ITEM)
        item.sticky = not item.sticky
        model.set(iter, self.COLUMN_STICKY_FLAG, item.sticky)

    def render_feed(self, feed):
        model = self._widget.get_model()
        selection = self._widget.get_selection()
        selection.unselect_all()
        model.clear()
        count = 0
        for item in feed.items:
            title = ""
            if item.title is not None and item.title != "":
                title = convert_entities(item.title)
            elif item.description is not None and item.description != "":
                desc = read_text(item.description, 50)
                title = convert_entities(desc)
                title += " ..."
            else:
                continue
            count += 1
            if item.seen:
                bold = pango.WEIGHT_NORMAL
            else:
                bold = pango.WEIGHT_BOLD
            iter = model.append()
            model.set(iter, self.COLUMN_TITLE, title, self.COLUMN_ITEM, item,
                      self.COLUMN_BOLD, bold,
                      self.COLUMN_STICKY_FLAG, item.sticky)

        self._widget.columns_autosize()
        return count

    def display_feed(self, feed, select_first = 1):
        if self._feed is not None:
            self._feed.signal_disconnect(straw.RefreshFeedDisplaySignal,
                                         self.feed_order_changed)
        self._feed = feed
        count = self.render_feed(feed)
        if count and select_first:
            self._widget.get_selection().select_path((0,))
        feed.signal_connect(straw.RefreshFeedDisplaySignal,
                            self.feed_order_changed)
        return

    def feed_order_changed(self, event):
        if event.sender is self._feed:
            selection = self._widget.get_selection()
            liststore, iter = selection.get_selected()
            path = liststore.get_path(iter)
            item = self._widget.get_model()[path[0]][self.COLUMN_ITEM]
            self.render_feed(event.sender)
            selection.select_path(event.sender.get_item_index(item))

    def display_empty_feed(self, feed):
        self._widget.get_model().clear()

    def item_selection_changed(self, selection):
        s = selection.get_selected()
        if s:
            model, iter = s
            if iter is None:
                return
            path = model.get_path(iter)
            index = path[0]
            item = model[index][self.COLUMN_ITEM]
            if not straw.main.is_visible_item(item):
                self.display_item(item)

    def mark_item(self, index):
        model = self._widget.get_model()
        path = (index,)
        try:
            iter = model.get_iter(path)
        except ValueError:
            return
        model.set_value(iter, self.COLUMN_BOLD, pango.WEIGHT_NORMAL)
        return

    def select_item(self, item):
        index = item.feed.get_item_index(item)
        path = (index,)
        self._widget.get_selection().select_path(path)
        self._widget.set_cursor(path, None, 0)
        self._widget.scroll_to_cell(path, None, 0, 0.5, 0)

    def display_item(self, item):
        self.mark_item(item.feed.get_item_index(item))
        straw.main.display_item(item)

class RSSQualityDisplay:
    def __init__(self, widget):
        widget_tree = glade.get_widget_tree(widget)
        self._widget = widget
        self._image = widget_tree.get_widget('rss_quality_image')
        self._tooltips = gtk.Tooltips()

    def show(self, broken):
        if broken == -1:
            self._widget.hide()
            return
        if broken:
            imagename = "face-sad"
            tooltip = _("The RSS feed contained broken XML")
        else:
            imagename = "face-happy"
            tooltip = _("The RSS feed contained valid XML")
        straw.main.show_image(self._image, imagename)
        self._tooltips.set_tip(self._widget, tooltip, tooltip)
        self._tooltips.enable()
        self._widget.show_all()

class OfflineToggle:
    def __init__(self, widget):
        self._widget = widget
        widget_tree = glade.get_widget_tree(widget)
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                widget_tree.signal_connect(key, getattr(self, key))
        self._widget.remove(self._widget.get_child())
        self._widget.add(gtk.Image())
        self._tooltips = gtk.Tooltips()
        straw.config.signal_connect(straw.OfflineModeChangedSignal,
                                    self.mode_changed)

    def show(self):
        if straw.config.offline:
            imagename = "offline"
            tooltip = _("Straw is currently offline. Click to work online.")
            self._widget.set_active(1)
        else:
            imagename = "online"
            tooltip = _("Straw is currently online. Click to work offline.")
            self._widget.set_active(0)
        straw.main.show_image(self._widget.get_child(), imagename)
        self._tooltips.set_tip(self._widget, tooltip, tooltip)
        self._tooltips.enable()
        self._widget.show_all()

    def on_offline_toggle_toggled(self, *args):
        active = self._widget.get_active()
        if active != straw.config.offline:
            straw.config.offline = not straw.config.offline

    def mode_changed(self, signal):
        self.show()

class FeedInfoDisplay:
    def __init__(self, widget):
        widget_tree = glade.get_widget_tree(widget)
        self._widget = widget
        self._state_indicator = widget_tree.get_widget('feed_info_arrow')
        self._drawer = widget_tree.get_widget('feed_info_drawer')
        self._switch_button = widget_tree.get_widget(
            'feed_info_visibility_button')
        self._title_display = widget_tree.get_widget('feed_info_title')
        self._link_display = widget_tree.get_widget('feed_info_link')
        self._description_display = widget_tree.get_widget(
            'feed_info_description')
        self._copyright_display = widget_tree.get_widget('feed_info_copyright')
        self._tooltips = gtk.Tooltips()
        self._visible = 0
        self.set_indicator(self._visible)

    def set_indicator(self, visible):
        self._state_indicator.set((gtk.ARROW_RIGHT, gtk.ARROW_DOWN)[visible],
                                  gtk.SHADOW_NONE)
        tooltip = (_("Show feed information"),
                   _("Hide feed information"))[visible]
        self._tooltips.set_tip(self._switch_button, tooltip, tooltip)
        return

    def show(self):
        self._drawer.show()

    def hide(self):
        self._drawer.hide()

    def switch_state(self):
        (self.show, self.hide)[self._visible]()
        self._visible = not self._visible
        self.set_indicator(self._visible)

    def display_feed(self, feed):
        self._widget.show()

        title = feed.channel_title
        self._title_display.set_text(title)
        if len(title):
            self._title_display.show()
        else:
            self._title_display.hide()

        description = feed.channel_description
        size = len(description)
        if size:
            if is_html(description):
                description = read_text(description, size)
            self._description_display.set_text(description)
            self._description_display.show()
        else:
            self._description_display.hide()

        copyright = feed.channel_copyright
        self._copyright_display.set_text(copyright)
        if len(copyright):
            self._copyright_display.show()
        else:
            self._copyright_display.hide()

        link = feed.channel_link
        self._link_display.set_url(link)
        self._link_display.set_property("label", link)
        if len(link):
            self._link_display.show()
        else:
            self._link_display.hide()

class FindResultList:
    COLUMN_TITLE = 0
    COLUMN_FEED = 1
    COLUMN_ITEM = 2

    def __init__(self, widget):
        widget_tree = glade.get_widget_tree(widget)
        self._treeview = widget_tree.get_widget('find_results_treeview')
        self._countlabel = widget_tree.get_widget('find_results_count_display')
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                              gobject.TYPE_PYOBJECT)
        self._treeview.set_model(model)
        self._treeview.set_rules_hint(gtk.TRUE)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Title', renderer, text=self.COLUMN_TITLE)
        self._treeview.append_column(column)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Feed', renderer, text=self.COLUMN_FEED)
        self._treeview.append_column(column)
        self._treeview.get_selection().connect("changed", self.item_selection_changed)
        self._find_interrupted = 0

    def render_results(self, items):
        model = self._treeview.get_model()
        model.clear()
        self._countlabel.set_text(_('%d items found') % len(items))
        i = 0
        for item in items:
            if self._find_interrupted:
                self._find_interrupted = 0
                break
            i += 1
            iter = model.append()
            title = ""
            if item.title is not None:
                title = item.title[:50]
            if title == "":
                title = read_text(item.description, 50) + "..."
            model.set(iter, self.COLUMN_TITLE, title,
                      self.COLUMN_FEED, item.feed.title,
                      self.COLUMN_ITEM, item)
            if i % 10 == 0:
                # gtk bug avoidance?
                # without threads_*, after returning from the timeout
                # callback everything will lock up
                gtk.threads_enter()
                while gtk.events_pending():
                    gtk.mainiteration(gtk.FALSE)
                gtk.threads_leave()

    def item_selection_changed(self, selection):
        s = selection.get_selected()
        if s:
            model, iter = s
            if iter is None:
                return
            path = model.get_path(iter)
            index = path[0]
            item = model[index][self.COLUMN_ITEM]
            straw.main.display_item_unselected(item)

    def interrupt_find(self, signal):
        self._find_interrupted = 42

    def register_find_dialog(self, fd):
        fd.signal_connect(straw.FindInterruptSignal, self.interrupt_find)

class MainWindow:
    COLUMN_NAME = 0
    COLUMN_UNREAD = 1
    COLUMN_STATUS_DISPLAY = 2
    COLUMN_BOLD = 3
    COLUMN_ID = 4
    COLUMN_STATUS_FLAG = 5
    COLUMN_STATUS_PIXBUF = 6

    def __init__(self, xml):
        self._item_view = ItemView(xml.get_widget('item_view_container'))
        self._rss_quality_display = RSSQualityDisplay(
            xml.get_widget('rss_quality_eventbox'))
        self._exception_view = ExceptionView(
            xml.get_widget('feed_exception_frame'),
            xml.get_widget('feed_exception_label'))
        self._feed_info_view = FeedInfoDisplay(
            xml.get_widget('feed_info_frame'))
        self._find_results_view = FindResultList(
            xml.get_widget('find_results_treeview'))
        self._offline_toggle = OfflineToggle(
            xml.get_widget('offline_toggle'))
        self._offline_toggle.show()
        self.create_ui(xml)
        # status messages are queued and displayed with a half a second
        # timeout, so the user can see what's happening. the timeout is only
        # set if there's stuff in the queue so we won't be doing useless
        # calls all the time when there's nothing to show
        self._status_messages = []

    def get_find_results_view(self):
        return self._find_results_view

    def on_straw_main_destroy_event(self, *args):
        return straw.main.quit()

    def on_straw_main_delete_event(self, *args):
        return straw.main.quit()

    def on_straw_main_configure_event(self, widget, event, *args):
        def check_size((width, height, widget)):
            if width == widget.allocation.width and height == widget.allocation.height:
                straw.config.main_window_size = (width, height)
        if event.width != widget.allocation.width or event.height != widget.allocation.height:
            gtk.timeout_add(1000, check_size, (
                (event.width, event.height, widget)))

    def on_main_main_pane_size_allocate(self, widget, *args):
        def check_position((position, widget)):
            if position == widget.get_position():
                straw.config.main_pane_position = position
        pos = widget.get_position()
        if pos != straw.config.main_pane_position:
            gtk.timeout_add(1000, check_position, (pos, widget))

    def on_main_sub_pane_size_allocate(self, widget, *args):
        def check_position((position, widget)):
            if position == widget.get_position():
                straw.config.sub_pane_position = position
        pos = widget.get_position()
        if pos != straw.config.sub_pane_position:
            gtk.timeout_add(1000, check_position, (pos, widget))

    def on_toolbar_preferences_show_button_clicked(self, *args):
        straw.main.show_preferences_dialog()
        return

    def on_toolbar_poll_button_clicked(self, button, *args):
        straw.poll_manager.poll()
        return

    def on_toolbar_find_button_clicked(self, button, *args):
        self.show_find_mode()

    def on_subscribe_new_feed_button_clicked(self, button, *args):
        straw.main.show_subscribe_dialog()

    def on_menu_about_activate(self, menuitem, *args):
        whatis = "Straw is a desktop news aggregator for GNOME"
        gnome.ui.About(
            straw.APPNAME, straw.VERSION, "Copyright (c) 2002-2003 Juri Pakaste", whatis,
            ["Juri Pakaste <juri@iki.fi>",
             "Jan Alonzo <jmalonzo@spaceants.org>",
             "rssparser.py and rssfinder.py by Mark Pilgrim",
             "Icons by Jakub 'jimmac' Steiner and Juri Pakaste"],
            [],
            "Juri Pakaste <juri@iki.fi>\n"
            "Martin Steldinger <tribble@hanfplantage.de>\n"
            "David Rousseau <boiteaflood@wanadoo.fr>\n"
            "Sergei Vavinov <svv@cmc.msu.ru>\n"
            u"Terje R\xf8sten <terjeros@phys.ntnu.no>",
            gtk.gdk.pixbuf_new_from_file(straw.main._libdir +
                                         "/straw.png")).show()

    def on_menu_quit_activate(self, *args):
        straw.main.quit()

    def on_menu_preferences_activate(self, *args):
        straw.main.show_preferences_dialog()
        return

    def on_menu_poll_all_activate(self, *args):
        straw.poll_manager.poll()

    def on_menu_poll_selected_activate(self, *args):
        straw.poll_manager.poll(straw.main.get_visible_feed())

    def on_menu_next_activate(self, *args):
        straw.main.display_next_item()
        return

    def on_menu_previous_activate(self, *args):
        straw.main.display_previous_item()

    def on_menu_scroll_next_activate(self, *args):
        if not self._item_view.scroll_down():
            straw.main.display_next_unread_item()

    def on_menu_next_feed_activate(self, *args):
        straw.main.display_next_feed()
        return

    def on_menu_previous_feed_activate(self, *args):
        straw.main.display_previous_feed()
        return

    def on_menu_next_feed_unread_activate(self, *args):
        straw.main.display_next_unread_feed()
        return

    def on_menu_feed_information_activate(self, *args):
        self._feed_info_view.switch_state()

    def on_menu_merge_subscriptions_activate(self, *args):
        straw.main.show_subscriptions_merge_dialog()

    def on_menu_catch_up_feed_activate(self, *args):
        straw.main.catch_up_feed()

    def on_menu_find_activate(self, *args):
        self.show_find_mode()

    def on_menu_import_subscriptions_activate(self, *args):
        straw.main.show_import_subscriptions_dialog()

    def on_menu_export_subscriptions_activate(self, *args):
        straw.main.show_export_subscriptions_dialog()

    def on_feed_info_visibility_button_clicked(self, button, *args):
        self._feed_info_view.switch_state()

    def on_straw_main_focus_in_event(self, *args):
        straw.dashboardfe.focus_in()

    def on_straw_main_focus_out_event(self, *args):
        straw.dashboardfe.focus_out()

    def feed_selection_changed(self, selection):
        s = selection.get_selected()
        if s:
            model, iter = s
            if iter is None:
                return
            path = model.get_path(iter)
            feed = straw.config.feeds[path[0]]
            if feed:
                straw.main.display_feed(feed)
        return

    def show_find_mode(self):
        self._mode_view_notebook.set_current_page(1)
        self._feed_selection_vbox.hide()
        straw.main.show_find_dialog()

    def hide_find_mode(self):
        self._mode_view_notebook.set_current_page(0)
        self._feed_selection_vbox.show()

    def show_find_results(self, items):
        self._find_results_view.render_results(items)

    def display_feed_unselected(self, feed, index = None):
        self._menu_merge_subscriptions.set_sensitive(
            feed.channel_subscriptions != "" or feed.channel_blogroll != "")
        if index is None:
            index = straw.config.feeds.index(feed)
        self._feed_list_view.set_cursor(
            (index,), self._feed_list_view.get_column(0), 0)
        self._feed_list_view.get_selection().select_path((index,))

    def display_feed(self, feed, select_first = 1):
        self._menu_merge_subscriptions.set_sensitive(
            feed.channel_subscriptions != "" or feed.channel_blogroll != "")
        if feed.error is None:
            self._exception_view.hide()
        else:
            self._exception_view.show_exception(feed.error)
        self._item_list_view.display_feed(feed, select_first)
        self._rss_quality_display.show(broken = feed.invalid_rss)
        self._feed_info_view.display_feed(feed)

    def display_empty_feed(self, feed):
        if feed.channel_subscriptions != "" or feed.channel_blogroll != "":
            self._menu_merge_subscriptions.set_sensitive(1)
        if feed.error is None:
            self._exception_view.hide()
        else:
            self._exception_view.show_exception(feed.error)
        self._item_list_view.display_empty_feed(feed)
        self._item_view.display_empty_feed(feed)

    def display_item_unselected(self, item):
        self._item_list_view.select_item(item)

    def display_item(self, item):
        self._item_view.display_item(item)

    def _display_status_messages(self):
        ql = len(self._status_messages)
        if ql:
            cid = self._status_view.get_context_id("asdf")
            self._status_view.pop(cid)
            m = self._status_messages.pop(0)
            if len(m):
                self._status_view.push(cid, m)
        if ql > 1:
            gtk.timeout_add(300, self._display_status_messages)

    def display_status_message(self, m):
        ql = len(self._status_messages)
        self._status_messages.append(m)
        if not ql:
            gtk.timeout_add(300, self._display_status_messages)

    def number_of_new(self, feed):
        nu = feed.number_of_unread
        return ("%s/%s" % (nu, feed.number_of_items), nu)

    def _select_feed_status_icon(self, feed):
        if feed.process_status is not straw.Feed.STATUS_IDLE:
            return (1, gtk.STOCK_EXECUTE)
        elif feed.error:
            return (1, gtk.STOCK_DIALOG_ERROR)
        return (0, None)

    def display_feeds(self, feeds, visible_feed):
        model = self._feed_list_view.get_model()
        selection = self._feed_list_view.get_selection()
        fc = 0
        mlines = len(model)
        # first iterate through either the currently visible feeds,
        # OR the feeds to be displayed, whichever is the smaller set
        while fc < mlines and fc < len(feeds):
            feed = feeds[fc]
            if model[fc][self.COLUMN_ID] != feed.id:
                try:
                    feed.signal_disconnect(
                        straw.ItemReadSignal, self.item_read)
                    feed.signal_disconnect(
                        straw.ItemsAddedSignal, self.feed_updated)
                    feed.signal_disconnect(
                        straw.AllItemsReadSignal, self.all_items_read)
                    feed.signal_disconnect(
                        straw.FeedPolledSignal, self.feed_updated)
                    feed.signal_disconnect(
                        straw.FeedStatusChangedSignal, self.feed_updated)
                except KeyError:
                    pass
                feed.signal_connect(straw.ItemReadSignal, self.item_read, fc)
                feed.signal_connect(straw.AllItemsReadSignal,
                                    self.all_items_read, fc)
                feed.signal_connect(
                    straw.ItemsAddedSignal, self.feed_updated, fc)
                feed.signal_connect(
                    straw.FeedPolledSignal, self.feed_updated, fc)
                feed.signal_connect(
                    straw.FeedStatusChangedSignal, self.feed_updated, fc)
                new_string, new = self.number_of_new(feed)
                weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new > 0]
                model[fc][self.COLUMN_NAME] = feed.title
                model[fc][self.COLUMN_UNREAD] = new_string
                model[fc][self.COLUMN_BOLD] = weight
                model[fc][self.COLUMN_ID] = feed.id
                status, pixbuf = self._select_feed_status_icon(feed)
                model[fc][self.COLUMN_STATUS_FLAG] = status
                if pixbuf:
                    model[fc][self.COLUMN_STATUS_PIXBUF] = pixbuf
                if feed is visible_feed:
                    selection.select_path((fc,))
            elif model[fc][self.COLUMN_NAME] != feed.title:
                model[fc][self.COLUMN_NAME] = feed.title
            fc += 1
        # if there were fewer feeds given as argument than currently displayed,
        # remove the extra rows
        if fc < mlines:
            treeiter = model.get_iter((fc,))
            while treeiter:
                next_iter = model.iter_next(treeiter)
                model.remove(treeiter)
                treeiter = next_iter
        # otherwise, add the extra rows and display the rest of the feeds
        # on them
        else:
            while fc < len(feeds):
                feed = feeds[fc]
                iter = model.append()
                new_string, new = self.number_of_new(feed)
                feed.signal_connect(straw.ItemReadSignal, self.item_read, fc)
                feed.signal_connect(straw.AllItemsReadSignal,
                                    self.all_items_read, fc)
                feed.signal_connect(straw.ItemsAddedSignal,
                                    self.feed_updated, fc)
                feed.signal_connect(straw.FeedPolledSignal,
                                    self.feed_updated, fc)
                feed.signal_connect(straw.FeedStatusChangedSignal,
                                    self.feed_updated, fc)
                weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new > 0]
                status, pixbuf = self._select_feed_status_icon(feed)
                model.set(iter, self.COLUMN_NAME, feed.title,
                          self.COLUMN_UNREAD, new_string,
                          self.COLUMN_BOLD, weight,
                          self.COLUMN_ID, feed.id,
                          self.COLUMN_STATUS_FLAG, status)
                if pixbuf:
                    model.set(iter, self.COLUMN_STATUS_PIXBUF, pixbuf)
                if feed is visible_feed:
                    selection.select_iter(iter)
                fc += 1
        self._feed_list_view.columns_autosize()
        self.update_total_unread_indicator()

    def update_total_unread_indicator(self):
        uritems = urfeeds = 0
        for ur in [f.number_of_unread for f in straw.config.feeds]:
            if ur:
                uritems += ur
                urfeeds += 1
        if urfeeds > 1: feedstr = "feeds"
        else: feedstr = "feed"
        map = {'uritems': uritems, 'urfeeds': urfeeds, 'feedstr': feedstr}
        self._window.set_title(('%s - ' % straw.APPNAME) +
                               _('%(uritems)d unread in %(urfeeds)d %(feedstr)s') %
                               map )

    def all_items_read(self, signal, feed_index):
        model = self._feed_list_view.get_model()
        new = self.number_of_new(signal.sender)
        weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
        model[feed_index][self.COLUMN_UNREAD] = new[0]
        model[feed_index][self.COLUMN_BOLD] = weight

        for index, item in signal.changed:
            self._item_list_view.mark_item(index)
        self._feed_list_view.queue_draw()
        self.update_total_unread_indicator()

    def item_read(self, signal, feed_index):
        model = self._feed_list_view.get_model()
        new = self.number_of_new(signal.sender.feed)
        weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
        model[feed_index][self.COLUMN_UNREAD] = new[0]
        model[feed_index][self.COLUMN_BOLD] = weight
        self._item_list_view.mark_item(
            signal.sender.feed.get_item_index(signal.sender))
        self._feed_list_view.queue_draw()
        self.update_total_unread_indicator()

    def feed_updated(self, signal, feed_index):
        model = self._feed_list_view.get_model()
        new = self.number_of_new(signal.sender)
        weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
        model[feed_index][self.COLUMN_UNREAD] = new[0]
        model[feed_index][self.COLUMN_BOLD] = weight
        status, pixbuf = self._select_feed_status_icon(signal.sender)
        model[feed_index][self.COLUMN_STATUS_FLAG] = status
        if pixbuf:
            model[feed_index][self.COLUMN_STATUS_PIXBUF] = pixbuf
        self._feed_list_view.queue_draw()
        if signal.sender is straw.main.get_visible_feed():
            self.display_feed(signal.sender)
        self.update_total_unread_indicator()

#     def items_added(self, signal, feed_index):
#         model = self._feed_list_view.get_model()
#         new = self.number_of_new(signal.sender)
#         weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
#         model[feed_index][self.COLUMN_UNREAD] = new[0]
#         model[feed_index][self.COLUMN_BOLD] = weight
#         model[feed_index][self.COLUMN_ERROR_FLAG] = signal.sender.error is not None
#         self._feed_list_view.queue_draw()
#         if straw.config.feeds[feed_index] is straw.main.get_visible_feed():
#             self.display_feed(straw.config.feeds[feed_index])
#         self.update_total_unread_indicator()

    def create_feed_selection_columns(self, feed_list_view):
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Title', renderer, text=self.COLUMN_NAME,
                                    weight=self.COLUMN_BOLD)
        feed_list_view.append_column(column)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('New', renderer, text=self.COLUMN_UNREAD,
                                    weight=self.COLUMN_BOLD)
        feed_list_view.append_column(column)
        renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('Status', renderer,
                                    stock_id=self.COLUMN_STATUS_PIXBUF,
                                    visible=self.COLUMN_STATUS_FLAG)
        feed_list_view.append_column(column)
        return

    def create_ui(self, xml):
        self._window = xml.get_widget('straw_main')
        self._window.set_default_size(*straw.config.main_window_size)
        xml.get_widget('main_main_pane').set_position(
            straw.config.main_pane_position)
        xml.get_widget('main_sub_pane').set_position(
            straw.config.sub_pane_position)
        self._window.show()
        while gtk.events_pending():
            gtk.mainiteration(gtk.FALSE)
        feed_list_view = xml.get_widget('feed_selection_treeview')
        self._mode_view_notebook = xml.get_widget('mode_view_notebook')
        self._feed_selection_vbox = xml.get_widget('feed_selection_vbox')
        self._status_view = xml.get_widget("main_statusbar")
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                xml.signal_connect(key, getattr(self, key))
        xml.get_widget('feed_selection_treeview').get_selection().connect(
            "changed", self.feed_selection_changed)
        model = gtk.ListStore(
            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_OBJECT,
            gobject.TYPE_INT,
            gobject.TYPE_INT, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        feed_list_view.set_model(model)
        feed_list_view.set_rules_hint(gtk.TRUE)
        feed_list_view.set_search_column(0)
        self.create_feed_selection_columns(feed_list_view)
        self._feed_list_view = feed_list_view
        self._menu_merge_subscriptions = xml.get_widget(
            'menu_merge_subscriptions')
        self._menu_merge_subscriptions.set_sensitive(0)
        self._item_list_view = ItemList(
            xml.get_widget('item_selection_treeview'))

        self._window.drag_dest_set(
            gtk.DEST_DEFAULT_ALL,
            [('_NETSCAPE_URL', 0, 0), ('text/uri-list ', 0, 1),
             ('x-url/http', 0, 2)],
            gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)

        self._window.show()
        return

    def get_window(self):
        return self._window

    def on_straw_main_drag_data_received(self, w, context, x, y, data, info, time):
        if data and data.format == 8:
            straw.main.show_subscribe_dialog(data.data)
            context.finish(gtk.TRUE, gtk.FALSE, time)
        else:
            context.finish(gtk.FALSE, gtk.FALSE, time)

    def disable_feed_selection(self):
        self._feed_list_view.set_sensitive(0)

    def enable_feed_selection(self):
        self._feed_list_view.set_sensitive(1)

